dojo.require('dojo.data.ItemFileWriteStore');

dojo.provide('ibm.data.RESTItemStore');

dojo.declare('ibm.data.RESTItemStore', dojo.data.ItemFileWriteStore, {
	
	saveURL:'',
	oldContent:'{label:"x",identifier:"x",items:[]}',
	useIdentifier:false,
	ignoredFields:'', 		// Use ignoredFields to prevent changes in specified fields from triggering a POST on _saveEverything 
	
	constructor: function(keywordParameters){
		this.saveURL = keywordParameters.saveURL;
		this.useIdentifier = keywordParameters.useIdentifier;
		this.ignoredFields = keywordParameters.ignoredFields;
 	},
 	

	_saveEverything: function(saveCompleteCallback, saveFailedCallback, newFileContentString) {
	
		var oldData = dojo.fromJson(this.oldContent);
		var newData = dojo.fromJson(newFileContentString);
		
		var mod = false;

		// First, iterate through old items.  If an old item has
		// the same id as a new item, check for modifications.  If
		// the old id is not found, remove it.
		
		for (var o=0;o<oldData.items.length;o++)
		{
			var id = oldData.items[o][oldData.identifier];
			var found = false;
			for (var n=0;n<newData.items.length;n++)
			{
				if (id == newData.items[n][newData.identifier])
				{
					found = true;
					if (!this._isMatch(oldData.items[o],newData.items[n]))
					{			
						var editData = this._makeBody(newData,newData.items[o]);
						var url = this.saveURL;
						if (this.useIdentifier)
							url+="/"+editData.items[0][editData.identifier];
						
						this._post(url,dojo.toJson(editData),saveCompleteCallback,saveFailedCallback);
						mod = true;
					}
				}
			}
			if (!found)
			{
				var url = this.saveURL;
				if (this.useIdentifier)
					url+="/"+oldData.items[o][oldData.identifier];
				
				this._delete(url,'',saveCompleteCallback,saveFailedCallback);
				mod = true;
			}
		}
			
		// Now check for new ids, to see if we have any put operations

		for (var n=0;n<newData.items.length;n++)
		{
			var id = newData.items[n][newData.identifier];
			var found = false;
			for (var o=0;o<oldData.items.length;o++)
			{
				if (id == oldData.items[o][oldData.identifier])
				{
					found = true;
				}
			}
			if (!found)
			{
				var editData = this._makeBody(newData,newData.items[n]);
				var url = this.saveURL;
				if (this.useIdentifier)
					url+="/"+editData.items[0][editData.identifier];
				
				this._put(url,dojo.toJson(editData),saveCompleteCallback,saveFailedCallback);
				mod = true;
			}
		}
		
		if (!mod)
			saveCompleteCallback();
		
		this.oldContent=newFileContentString;
	},
	
	_makeBody: function(newData,item)
	{
		var editData = dojo.fromJson(dojo.toJson(newData));
		editData.items = new Array();
		editData.items[0] = item;
		return editData;
	},
	
	_isMatch: function(item1,item2)
	{
		if (item1==null || item2==null)
			return true;
		
		if (typeof item1 == "object")
		{			
			for (var prop in item1)
			{
				if (!this._contains(this.ignoredFields,prop))
				{
					var val1 = item1[prop];
					var val2 = item2[prop];
					
					if (!this._isMatch(val1,val2))
						return false;
				}
			}
		}
		else{
			return (item1==item2);
		}
		return true;
	},
	
	_contains: function(list,item)
	{
		if (list==null)
			return false;
	
		var array = list.split(",");
		for (var a=0;a<array.length;a++){
			if (array[a].trim() == item){
				return true;
			}
		}
		return false;
	},
	
	_put: function(url,json,callback,errCallback)
	{
		dojo.xhrPut({
			url: url,
			putData: json,
			load: callback,
			error: errCallback
		});
	},
	
	_post: function(url,json,callback,errCallback)
	{
		dojo.xhrPost({
			url: url,
			postData: json,
			load: callback,
			error: errCallback
		});
	},
	
	_delete: function(url,json,callback,errCallback)
	{
		dojo.xhrDelete({
			url: url,
			deleteData: json,
			load: callback,
			error: errCallback
		});
	},
	
	_getItemsFromLoadedData: function(/* Object */ dataObject){
		this.inherited(arguments);
		this.oldContent = this._getNewFileContentString();
	}
});